/*
 * Copyright 2007 The Fornax Project Team, including the original
 * author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.sculptor.ext

import static extension org.sculptor.ext.Helper.*;
import sculptormetamodel.Application
import sculptormetamodel.Repository
import sculptormetamodel.Module
import sculptormetamodel.DomainObject
import java.util.List
import java.util.Collection
import sculptormetamodel.Attribute
import sculptormetamodel.Reference
import sculptormetamodel.NamedElement

class Helper {
	def static String javaFileName(String name) {
		name.replaceAll("\\.", "/") + ".java";
	}

	def static Collection<Repository> getAllRepositories(Application app) {
		app.getAllRepositories(true)
	}

	def static Collection<Repository> getAllRepositories(Module module) {
		(module.domainObjects as List<DomainObject>).filter[repository?.name != null].map[repository].sortBy[name];
	}

	def static Collection<Repository> getAllRepositories(Application app, boolean includeExternal) {
		(app.modules as List<Module>).map[it.getAllRepositories].flatten
			.filter[includeExternal || !aggregateRoot.module.external].sortBy[name];
	}

	// ********************************************************************************
	// Additional dummy method imlementations for DomainObjectProperties.xtend
	// ********************************************************************************
	def static String javaHeader() {
		"/* JAVA HEADER */"
	}
	def static String getDomainPackage(DomainObject d) {d.domainPackage;}
	def static List<Attribute> getAllAttributes(DomainObject d) {d.attributes}
	def static List<Reference> getAllReferences(DomainObject d) {d.references}
	def static boolean isEnumReference(Reference ref) {
		!ref.many && (ref.to.metaType == typeof(Enum));
	}
	def static Class<?> metaType(NamedElement ne) {
		JavaHelper::getJavaClass(ne);
	}
	def static boolean isUnownedReference(NamedElement elem) {
		false
	}
	def static boolean isUnownedReference(Reference ref) {
//		(isJpaProviderAppEngine() || nosql()) && ref.from.isPersistent() && !ref.transient && ref.to.hasOwnDatabaseRepresentation()
//			&& !ref.hasOwnedHint() && (!ref.from.getAggregate().contains(ref.to) || ref.hasUnownedHint());
		true
	}

	def static boolean nosql() {
		false
	}
	def static fw(String prop) {
		prop
	}

	// Embed XPand REM comments
	def static REM() {
		'/*'
	}
	def static ENDREM() {
		'*/'
	}
}